-------------------------------------------------------------
--	Filename:  AFE.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 3
--	Date last modified: 6/7/05
-- Inheritance: 	none
--
-- Description: AD9860/2 analog front-end interface driver.
-- The AD9860 is configured for a 64 MHz input reference clock, 64 Msamples/s dual A/D conversion,
-- and 128 Msamples/s dual D/A conversion with x2 interpolation. 
-- A/D samples are read at 64 Msamples/s on two parallel busses
-- D/A samples are written at 64 Msamples/s, time-muxed at 128 MHz on a single shared bus.
-- For best performances, all high-speed D/A samples bits should use the same FPGA I/O bank.
--
-- Code key assumption: AFE sampling rate of 64 Msamples/s (A/D and D/A) 
-- is always less than FPGA processing clock (CLK = 90+ MHz).
-- CLKOUT1# is made a global clock for better control timing.
--
-- Implementation: Uses one Global buffer.
-- 
-- Rev 3 6/6/05  AZ
-- switched to shared voltage reference for new COM-1200
-- switched to normal mode. 64 MHz clock input.
-- use AFE_CNTRL5 instead of CLKOUT1 (reason: same clock, saves global buffers).
--
-- Note: 12/4/09 AZ
-- Previous .ucf file was missing net AFE_CNTRL2. Affected aux adcs.
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- UNISIM library includes all Xilinx primitives (no longer any need to 
-- declare under the Components section. ISE 6.3.
Library UNISIM;
use UNISIM.vcomponents.all;

entity AFE is port ( 

	--// Clocks Resets
   CLK : in std_logic;				
		-- Master clock for this FPGA
		-- MUST BE GREATER THAN 64 MHz. (>= 90 MHz recommended).
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high
		
	--// AFE Interfaces 
	AFE_RX_DATA_A: in std_logic_vector(11 downto 0);
	AFE_RX_DATA_B: in std_logic_vector(11 downto 0);
		-- 12-bit high-speed ADC samples (AD9862)
		-- 10-bit high-speed ADC samples (AD9860)
		-- synchronous with AFE_CNTRL5. Read at falling edge of AFE_CNTRL5.
	AFE_TX_DATA: out std_logic_vector(13 downto 0);
	AFE_TXSYNC: out std_logic;
		-- 14-bit high-speed DAC samples (AD9862)
		-- 12-bit high-speed DAC samples (AD9860). 
		-- Two channels time-share the TX_DATA bus as per TXSYNC indication
		-- synchronous with CLKOUT2. Write at falling edge of CLKOUT2.
	AFE_CNTRL0: out std_logic;
	AFE_CNTRL1: out std_logic;
	AFE_CNTRL2: in std_logic;
	AFE_CNTRL3: out std_logic;
	AFE_CNTRL4: out std_logic;
	AFE_CNTRL5: in std_logic;
--	AFE_CNTRL6: in std_logic;
	AFE_CNTRL7: in std_logic;
	AFE_CNTRL8: out std_logic;
	AFE_CNTRL9: out std_logic;
  		-- Controls. 
		-- Serial port interface (SPI), to write and read from the ADC9860/2 internal 
		-- programming registers
		-- bit 0: RESETB (output)
		-- bit 1: SPI SCLK (output)
		-- bit 2: SPI SDO (input)
		-- bit 3: SPI SDIO (output)
		-- bit 4: SPI SEN (output)
		-- bit 5: 64 MHz clock (global buffer generated at top level)
		-- bit 6: CLKOUT1 (64 MHz input) unused (same as AFE_CNTRL5) (read AFE_RX_DATA_A/B at falling edge of CLKOUT1)
		-- bit 7: CLKOUT2 (128 MHz input) 
		-- bit 8: CLKSEL (output)
		-- bit 9: MODE/TxBLANK (output)
	AUX_SPI: out std_logic_vector(2 downto 0);
		-- Auxillary SPI, routed to a DAC for fast gain control
		-- bit 0: chip select #
		-- bit 1: clk
		-- bit 2: data out

	--// Controls
	RxPGA_A: in std_logic_vector(4 downto 0);
		-- receive channel A programmable gain. min 0 = 0 dB, max x14 = 20 dB gain
		-- set at power-up
	RxPGA_B: in std_logic_vector(4 downto 0);
		-- receive channel B programmable gain. min 0 = 0 dB, max x14 = 20 dB gain
		-- set at power-up
	RxPGA_SAMPLE_CLK: in std_logic;
	AUX_DAC_A: in std_logic_vector(7 downto 0);
	AUX_DAC_B: in std_logic_vector(7 downto 0);
	AUX_DAC_C: in std_logic_vector(7 downto 0);
	AUX_DAC_A_SAMPLE_CLK: in std_logic;
	AUX_DAC_B_SAMPLE_CLK: in std_logic;
	AUX_DAC_C_SAMPLE_CLK: in std_logic;
		-- write 8-bit AUX_DACs values to AD9860 control registers.
		-- Format: straight binary.
		-- takes about 1.5 usec if the AFE process is not writing to/reading from the AD9860
 	Tx_NCO: in std_logic_vector(24 downto 0);
		-- fine frequency tuning. 2's complement. format fc = Tx_NCO * 128 MHz/2^26
		-- Fine tuning NCO is disabled if Tx_NCO = 0 (saves power).

	--// Monitoring 4 auxillary ADC values
	-- Data is read continuously. As data is read in several byte-size fragments, 
	-- integrity is maintained only at the time when the SAMPLE_CLK is '1'.
	-- Conversion occurs once per 4 AUX ADC read cycle (12 usec). 
	-- 1/2 mux selection alternates every other conversion.
	-- Thus all 4 conversions (A1,A2,B1,B2) occur every 24 usecs.
	AUX_ADC_A1: out std_logic_vector(9 downto 0);
	AUX_ADC_A2: out std_logic_vector(9 downto 0);
	AUX_ADC_B1: out std_logic_vector(9 downto 0);
	AUX_ADC_B2: out std_logic_vector(9 downto 0);
	AUX_ADC_A1_SAMPLE_CLK: out std_logic;
	AUX_ADC_A2_SAMPLE_CLK: out std_logic;
	AUX_ADC_B1_SAMPLE_CLK: out std_logic;
	AUX_ADC_B2_SAMPLE_CLK: out std_logic;

	--// Received data stream
	-- Synchronous with CLK
	RX_DATA_SAMPLE_CLK: out std_logic;
	RX_DATA_A: out std_logic_vector(11 downto 0);
	RX_DATA_B: out std_logic_vector(11 downto 0);
		-- 12 bit precision (AD9862)
		-- 10 bit precision (AD9860). Ignore 2 LSBs
		-- format: unsigned
	--// Transmit data stream
	-- Synchronous with CLK
	--TX_DATA_SAMPLE_CLK: in std_logic;
	TX_DATA_A: in std_logic_vector(13 downto 0);
	TX_DATA_B: in std_logic_vector(13 downto 0);
		-- 14 bit precision (AD9862)
		-- 12 bit precision (AD9860). 2 LSBs ignored.
		-- format: unsigned
		-- Resampled at 64 MSamples/s. TX)DATA_A/B must be stable between TX_DATA_SAMPLE_CLK samples


  --// Test point
  S1_3: out std_logic_vector(7 downto 0);
  S1_3_CLK: out std_logic;
  S2_3: out std_logic_vector(7 downto 0);
  S2_3_CLK: out std_logic;
  TP: out std_logic_vector(10 downto 1)

);
end entity;

architecture behavioral of AFE is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
component SPI is port ( 
	--// Clocks Resets
   CLK : in std_logic;				-- Master clock for this FPGA
	CLK_DIV: in std_logic;			
		-- slow down master clock as maximum SPI speed is 16 MHz
		-- All processes within this module act only when CLK_DIV = '1'.
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high
		
	--// control
	ADDR: in std_logic_vector(5 downto 0);
		-- This information can change immediately after the request is made
	DATA_OUT: in std_logic_vector(7 downto 0);
		-- byte to be written to specified register
		-- This information can change immediately after the request is made
	WRITE_REQ: in std_logic;
		-- 1 CLK_DIV pulse indicating that the DATA_OUT is to be written out
		-- to the specified register.
	READ_REQ: in std_logic;
		-- 1 CLK_DIV pulse indicating a read request
	DATA_IN: out std_logic_vector(7 downto 0);
		-- byte read from the specified register
	DATA_IN_CLK: out std_logic;
		-- 1 CLK pulse indicating that DATA_IN is valid.
	BUSY: out std_logic;
		-- '0' to indicate that the SPI interface is available. Goes to '1'
		-- immediately after WRITE_REQ and READ_REQ until command is complete.

  --// SPI interface
  SEN: out std_logic;
  SCLK: out std_logic;
  SDIO: out std_logic;
  SDO: in std_logic
 );
end component;

--------------------------------------------------------
--     ATTRIBUTES
--------------------------------------------------------
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------

--// Constants
signal ZERO: std_logic;
signal ONE: std_logic;
signal ZERO16: std_logic_vector(15 downto 0);

--// Initialization sequence controller
signal SEQUENCE_COUNTER: std_logic_vector(6 downto 0);
constant SEQUENCE_COUNTER_MAX: std_logic_vector(6 downto 0) := "1000000";	
	-- upper register to be initialized is register 63
	-- when finished initializing, the SEQUENCE_COUNTER will stay in idle mode at 64
signal INITIAL_CONFIGURATION_COMPLETE: std_logic;
signal CLK_DIV: std_logic;
signal CLK_DIV_COUNTER: std_logic_vector(3 downto 0);	
constant CLK_DIV_FACTOR: std_logic_vector(3 downto 0) := "0011";	
	-- slow down SPI clock to at most 16 MHz.
	-- divides the CLK_P processing clock (120 MHz nominal) by (CLK_DIV_FACTOR + 1)
	-- to generate CLK_DIV.
	-- Further divided by two, as it takes two steps to create the SCLK.

--// SPI controls
signal SPI_ADDR: std_logic_vector(5 downto 0);
signal SPI_DATA_OUT: std_logic_vector(7 downto 0);
signal SPI_WRITE_REQ: std_logic;
signal SPI_READ_REQ: std_logic;
signal SPI_DATA_IN: std_logic_vector(7 downto 0);
signal SPI_DATA_IN_CLK: std_logic;
signal SPI_BUSY: std_logic;

--// High-speed ADC (receiver) section
signal AFE_RX_DATA_A_D: std_logic_vector(15 downto 0);
signal AFE_RX_DATA_B_D: std_logic_vector(15 downto 0);
signal RX_DATA_A_E: std_logic_vector(15 downto 0);
signal RX_DATA_B_E: std_logic_vector(15 downto 0);
signal RX_WPTR: std_logic_vector(7 downto 0);
signal RX_WPTR_P: std_logic_vector(7 downto 0);
signal RX_WPTR_0_D: std_logic;
signal RX_WPTR_0_D2: std_logic;
signal RX_RPTR: std_logic_vector(7 downto 0);
signal RX_RPTR_INC: std_logic_vector(7 downto 0);
signal RX_BUF_EMPTY : std_logic;
signal RX_DATA_READ_REQ: std_logic;
signal CLKOUT2B: std_logic; 
signal AFE_CNTRL5_N: std_logic; 
signal RX_GAIN_CONFIG_PENDING: std_logic;


--// High-speed DAC (transmitter) section
signal TX_SELECT: integer range 0 to 2;
signal TX_SELECT0: std_logic;
signal TX_SELECT0_D: std_logic;
signal TX_SELECT1: std_logic;
signal TX_SELECT1_D: std_logic;
signal TX_SELECT2: std_logic;
signal TX_SELECT2_D: std_logic;
signal TX_DATA_A_0: std_logic_vector(13 downto 0);
signal TX_DATA_A_1: std_logic_vector(13 downto 0);
signal TX_DATA_A_2: std_logic_vector(13 downto 0);
signal TX_DATA_B_0: std_logic_vector(13 downto 0);
signal TX_DATA_B_1: std_logic_vector(13 downto 0);
signal TX_DATA_B_2: std_logic_vector(13 downto 0);
signal AFE_TXSYNC_LOCAL: std_logic;
signal AFE_TX_DATA_A: std_logic_vector(13 downto 0);
signal AFE_TX_DATA_B: std_logic_vector(13 downto 0);
signal TX_NCO_ENABLED: std_logic;
signal Tx_NCO_SIGN: std_logic;
signal Tx_NCO_ABS: std_logic_vector(23 downto 0);

--// AUX ADCs
signal AUX_DAC_A_CONFIG_PENDING: std_logic;
signal AUX_DAC_B_CONFIG_PENDING: std_logic;
signal AUX_DAC_C_CONFIG_PENDING: std_logic;

--// AUX DACs
signal AUX_ADC_CONVERT_REQ_PENDING: std_logic;
signal AUX_ADC_1_2N: std_logic;
signal HOLD_AUX_ADC_READ: std_logic;
signal HOLD_AUX_ADC_READ_COUNTER: std_logic_vector(8 downto 0);

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
-- constants
ZERO <= '0';
ONE <= '1';
ZERO16 <= (others => '0');

BUFG_010: BUFG port map(I => AFE_CNTRL7, O => CLKOUT2B);	
	-- 128 MHz input clock for tx signals

AFE_CNTRL5_N <= not AFE_CNTRL5;

-- divide reference clock by 16 (120 MHz / 8 = 15 MHz < maximum SPI clock of 16 MHz)
CLK_DIV_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV_COUNTER = CLK_DIV_FACTOR) then
			CLK_DIV_COUNTER <= (others => '0');
			CLK_DIV <= '1';
		else
			CLK_DIV_COUNTER <= CLK_DIV_COUNTER + 1;
			CLK_DIV <= '0';
		end if;
		
	end if;
end process;

-- Basic AFE control signals
		-- bit 0: RESETB
		-- bit 1: SPI SCLK -
		-- bit 2: SPI SDO -
		-- bit 3: SPI SDIO -
		-- bit 4: SPI SEN -
		-- bit 5: 64 MHz clock
		-- bit 6: CLK_OUT1
		-- bit 7: CLK_OUT2
		-- bit 8: CLKSEL
		-- bit 9: MODE/TxBLANK
--AFE_CNTRL(2) <= 'Z';
--AFE_CNTRL(5) <= 'Z';
--AFE_CNTRL(7) <= 'Z';
AFE_CNTRL8 <= '1'; 	-- CLKSEL: 128 MHz to 64 MHz when generating AFE_CNTRL5 (receive sampling clock)
AFE_CNTRL9 <= '0';	-- select normal timing mode at reset.

-- Enable Tx NCO if the Tx_NCO_ABS is not null
TX_NCO_ENABLED <= '1' when (Tx_NCO /= 0) else '0';
-- convert signed format to abs and sign
Tx_NCO_SIGN <= Tx_NCO(24);
Tx_NCO_ABS_GEN_001: process(Tx_NCO)
begin
	if(Tx_NCO = "1000000000000000000000000") then
		-- most negative
		Tx_NCO_ABS <= x"FFFFFF";	
	elsif(Tx_NCO(24) = '1') then
		-- regular negative
		Tx_NCO_ABS <= not(Tx_NCO(23 downto 0)) + 1;	
	else
		-- positive
		Tx_NCO_ABS <= Tx_NCO(23 downto 0);	
	end if;
end process;



-- Sequence AD9860 configuration registers
-- Sequence counter and control register address
-- SEQUENCE_COUNTER first counts from 96 to 127 (activating RESETB)
-- then rolls over to zero and start the initialization sequence.
-- stops at SEQUENCE_COUNTER_MAX when initialization is complete.
SEQUENCE_COUNTER_GEN_001: process(ASYNC_RESET, CLK)
begin
	if(ASYNC_RESET = '1') then
		SEQUENCE_COUNTER <= "1100000";	
			-- start above SEQUENCE_COUNTER_MAX, activate RESETB until reaching 0
	elsif rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(SEQUENCE_COUNTER = 0) and (SPI_BUSY = '0') then
				-- initial configuration.
				-- get started as soon as the SPI interface is available
				SEQUENCE_COUNTER <= "0000001";
				SPI_ADDR <= SEQUENCE_COUNTER(5 downto 0);
		  	elsif(SEQUENCE_COUNTER /= SEQUENCE_COUNTER_MAX) and (SPI_BUSY = '0') then
				-- initial configuration
				-- up to SEQUENCE_COUNTER_MAX registers to configure.
				SEQUENCE_COUNTER <= SEQUENCE_COUNTER + 1;
				SPI_ADDR <= SEQUENCE_COUNTER(5 downto 0);
			elsif(INITIAL_CONFIGURATION_COMPLETE = '1') then
				-- initial configuration is complete. Now ready to accept individual 
				-- configuration requests. Arbitration among multiple possible requests.
				if(RX_GAIN_CONFIG_PENDING = '1') and (SPI_BUSY = '0') and (SPI_ADDR /= 2) then
					-- RX_GAIN configuration request. channel A
					SPI_ADDR <= "000010";	-- register 2
				elsif(RX_GAIN_CONFIG_PENDING = '1') and (SPI_BUSY = '0') and (SPI_ADDR = 2) then
					-- RX_GAIN configuration request. channel B
					SPI_ADDR <= "000011";	-- register 3
				elsif(AUX_ADC_CONVERT_REQ_PENDING = '1') and (SPI_BUSY = '0') then
					-- AUX ADC configuration request. 
					SPI_ADDR <= "100010";	-- register 34
				elsif (AUX_DAC_A_CONFIG_PENDING = '1') and (SPI_BUSY = '0') then
					-- single Aux DAC A configuration request. Not part of the initial configuration sequence.
					SPI_ADDR <= "100100";	-- register 36
				elsif (AUX_DAC_B_CONFIG_PENDING = '1') and (SPI_BUSY = '0') then
					-- single Aux DAC B configuration request. Not part of the initial configuration sequence.
					SPI_ADDR <= "100101";	-- register 37
				elsif (AUX_DAC_C_CONFIG_PENDING = '1') and (SPI_BUSY = '0') then
					-- single Aux DAC C configuration request. Not part of the initial configuration sequence.
					SPI_ADDR <= "100110";	-- register 38

				--// when not writing configurations, continuously read registers such as AUX ADCs
				-- Hold the process after starting ADC conversion.
				elsif (SPI_BUSY = '0') then
--				elsif (SPI_BUSY = '0') and (HOLD_AUX_ADC_READ = '0') then
					if(SPI_ADDR < 26) or (SPI_ADDR >= 33) then
						SPI_ADDR <= "011010";	-- register 26
					else 
						SPI_ADDR <= SPI_ADDR + 1; -- register range 27 - 33
					end if;
				end if;
			end if;
		end if;
	end if;
end process;

-- Initial configuration is complete when SEQUENCE_COUNTER = SEQUENCE_COUNTER_MAX
INITIAL_CONFIGURATION_COMPLETE <= '1' when (SEQUENCE_COUNTER = SEQUENCE_COUNTER_MAX) else '0';

-- Place the ADC into RESET (active low) when counter is 127
-- Minimum RESETB duration is  5 clock widths min as per specs page 4.
-- Use sequence counter range 96-127 (as there are no registers in that range)
RESETB_GEN: process(ASYNC_RESET, CLK)
begin
	if(ASYNC_RESET = '1') then
		AFE_CNTRL0 <= '1';
	elsif rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(SEQUENCE_COUNTER(6 downto 5) = "11") then
				AFE_CNTRL0 <= '0';	-- reset active
			else
				AFE_CNTRL0 <= '1';	-- reset inactive
			end if;
		end if;
	end if;
end process;

-- write to selected control registers 
-- SPI_WRITE_REQ is a wide pulse (1 CLK_DIV period long).
CONTROL_REG_DATA_OUT_001: process(CLK, RxPGA_A, RxPGA_B)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if(SPI_BUSY = '0') then
	--(DEFAULT)			if(SEQUENCE_COUNTER = 0)  then
					-- write to control register 0
					-- no software reset (just did a hardware reset), use dedicated serial lines for i/o
					-- MSB first
	--				SPI_DATA_OUT <= x"00";
	--				SPI_WRITE_REQ <= '1';
	--(DEFAULT)			elsif(SEQUENCE_COUNTER = 1)  then
					-- register 1: no rx power down
	--				SPI_DATA_OUT <= x"00";
	--				SPI_WRITE_REQ <= '1';
				if(SEQUENCE_COUNTER = 2) then
					-- write to control register2
					-- programmable gain, rx channel A
					SPI_DATA_OUT <= "000" & RxPGA_A;
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 3) then
					-- write to control register 3
					-- programmable gain, rx channel B
					SPI_DATA_OUT <= "000" & RxPGA_B;
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 4) then
					-- register 4: high speed duty cycle enabled, 
					-- clk duty enabled, shared reference enabled (new COM-1200)
					SPI_DATA_OUT <= x"07";
					SPI_WRITE_REQ <= '1';
	--(DEFAULT)			elsif(SEQUENCE_COUNTER = 5)  then
					-- register 5: reclocking rx data with CLKOUT1,straight binary format, no mux
	--				SPI_DATA_OUT <= x"00";
	--				SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 6)  then
					-- register 6: 2 channel, no hilbert, no decimate
					SPI_DATA_OUT <= x"08";
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 14)  then
					-- Register 14: programmable DAC gain A. Do not scale output current
					SPI_DATA_OUT <= x"80";
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 15)  then
					-- Register 15: programmable DAC gain B. Do not scale output current
					--SPI_DATA_OUT <= x"80"; -- same as register 14 above
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 16) then
					-- Register 16: TxPGA programmable gain. Maximum gain scaling to IOUTfsMAX
					SPI_DATA_OUT <= x"C8";	-- specification issue: 20 dB/0.1dB steps= 200, not 255 or 127 as per spec page 14.
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 18)  then
					-- Register 18: interleaved I/Q, reclock with CLKOUT2, straight binary format
					-- resample at rising edge of CLKOUT2.
					SPI_DATA_OUT <= x"01";	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 19)  then
					-- Register 19: enable both tx channels, x2 interpolation 
					SPI_DATA_OUT <= x"11";	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 20)  then
					-- Register 20: fine tune sign, enable fine tuning if Tx_NCO_ABS /= 0, 
					-- complex mix, coarse modulation disabled. 
					SPI_DATA_OUT <= "00" & Tx_NCO_SIGN & TX_NCO_ENABLED & "0000" ;	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 21)  then
					-- Register 21: NCO tuning word. LSB 
					SPI_DATA_OUT <= Tx_NCO_ABS(7 downto 0);	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 22)  then
					-- Register 22: NCO tuning word. 
					SPI_DATA_OUT <= Tx_NCO_ABS(15 downto 8);	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 23)  then
					-- Register 23: NCO tuning word. MSB 
					SPI_DATA_OUT <= Tx_NCO_ABS(23 downto 16);	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 24) then
					-- Register 24: DLL. x2. ADC x1, NO crystal input clock, DLL fast
					SPI_DATA_OUT <= x"49";	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 25) then
					-- Register 25: CLKOUT. undivided DLL output (128 MHz w/ 32MHz input), 
					-- CLKOUT1 not inverted, disabled,
					-- CLKOUT2 inverted, enabled
					SPI_DATA_OUT <= x"21";	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 34) then
					-- Register 34: AUX ADC controls. Aux SPI disabled, internal reference, 
					-- select A2 & B2, Start conversion  A & B
					SPI_DATA_OUT <= x"2D";	
					SPI_WRITE_REQ <= '1';
				elsif(SEQUENCE_COUNTER = 35) then
					-- Register 35: AUX ADC clock. Divide by 4 to keep clock below 20 MHz.
					SPI_DATA_OUT <= x"01";	
					SPI_WRITE_REQ <= '1';
			 	-- end of initial configuration section
				elsif (INITIAL_CONFIGURATION_COMPLETE = '1') then
					-- start of runtime phase. RX_GAIN, AUX ADC read and AUX DAC configuration
					-- initial configuration is complete. Now ready to accept individual 
					-- configuration requests
					if(RX_GAIN_CONFIG_PENDING = '1') and (SPI_ADDR /= 2) then
						-- write to control register2
						-- programmable gain, rx channel A
						SPI_DATA_OUT <= "000" & RxPGA_A;
						SPI_WRITE_REQ <= '1';
					elsif(RX_GAIN_CONFIG_PENDING = '1') and (SPI_ADDR = 2) then
						-- write to control register 3
						-- programmable gain, rx channel B
						SPI_DATA_OUT <= "000" & RxPGA_B;
						SPI_WRITE_REQ <= '1';
					elsif (AUX_ADC_CONVERT_REQ_PENDING = '1') then
						-- register 34: AUX ADC configuration request. 
						-- Start conversions for A and B. Select A1/2 based on AUX_ADC_1_2N
						SPI_DATA_OUT <= "001" & AUX_ADC_1_2N & "11" & AUX_ADC_1_2N & "1";	
						SPI_WRITE_REQ <= '1';
					elsif (AUX_DAC_A_CONFIG_PENDING = '1') then
						-- register 36: single Aux DAC A configuration request. Not part of the initial configuration sequence.
						-- Configuration is immediate upon completing serial write (by default configuration).
						SPI_DATA_OUT <= AUX_DAC_A;	
						SPI_WRITE_REQ <= '1';
					elsif (AUX_DAC_B_CONFIG_PENDING = '1') then
						-- register 37: single Aux DAC B configuration request. Not part of the initial configuration sequence.
						-- Configuration is immediate upon completing serial write (by default configuration).
						SPI_DATA_OUT <= AUX_DAC_B;	
						SPI_WRITE_REQ <= '1';
					elsif (AUX_DAC_C_CONFIG_PENDING = '1') then
						-- register 38: single Aux DAC C configuration request. Not part of the initial configuration sequence.
						-- Configuration is immediate upon completing serial write (by default configuration).
						SPI_DATA_OUT <= AUX_DAC_C;	
						SPI_WRITE_REQ <= '1';
					else
						SPI_WRITE_REQ <= '0';
					end if;
				else
					SPI_WRITE_REQ <= '0';
				end if;
			else
				SPI_WRITE_REQ <= '0';
			end if;
		end if;
	end if;
end process;

-- Read only when initial configuration is complete, 
-- no write request is pending and SPI engine is not busy.
-- SPI_READ_REQ is wide (1 CLK_DIV period long).
READ_REQ_GEN_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(CLK_DIV = '1') then	-- slow down SPI to less than 16 MHz
			if (INITIAL_CONFIGURATION_COMPLETE = '1') and 
				(RX_GAIN_CONFIG_PENDING = '0') and 
				(AUX_DAC_A_CONFIG_PENDING = '0') and 
				(AUX_DAC_B_CONFIG_PENDING = '0') and 
				(AUX_DAC_C_CONFIG_PENDING = '0') and 
				(AUX_ADC_CONVERT_REQ_PENDING = '0') and 
				(SPI_BUSY = '0') then 
				SPI_READ_REQ <= '1';
			else
				SPI_READ_REQ <= '0';
			end if;
		end if;
	end if;
end process;

--// AUX DAC SECTION ---------------
-- AUX_DAC_A configuration. Queue until configuration starts. 
AUX_DAC_A_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(AUX_DAC_A_SAMPLE_CLK = '1') then
			-- wait until SPI write is complete
			AUX_DAC_A_CONFIG_PENDING <= '1';
		elsif (INITIAL_CONFIGURATION_COMPLETE = '1') and (CLK_DIV = '1') and (SPI_ADDR = 36) and (SPI_BUSY = '0')  then
			-- just started configuring register 36. Ready for another word.
			AUX_DAC_A_CONFIG_PENDING <= '0';
		end if;
 	end if;
end process;

-- AUX_DAC_B configuration. Queue until configuration starts. 
AUX_DAC_B_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(AUX_DAC_B_SAMPLE_CLK = '1') then
			-- wait until SPI write is complete
			AUX_DAC_B_CONFIG_PENDING <= '1';
		elsif (INITIAL_CONFIGURATION_COMPLETE = '1') and (CLK_DIV = '1') and (SPI_ADDR = 37) and (SPI_BUSY = '0')  then
			-- just started configuring register 37. Ready for another word.
			AUX_DAC_B_CONFIG_PENDING <= '0';
		end if;
 	end if;
end process;

-- AUX_DAC_C configuration. Queue until configuration starts. 
AUX_DAC_C_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(AUX_DAC_C_SAMPLE_CLK = '1') then
			-- wait until SPI write is complete
			AUX_DAC_C_CONFIG_PENDING <= '1';
		elsif (INITIAL_CONFIGURATION_COMPLETE = '1') and (CLK_DIV = '1') and (SPI_ADDR = 38) and (SPI_BUSY = '0')  then
			-- just started configuring register 38. Ready for another word.
			AUX_DAC_C_CONFIG_PENDING <= '0';
		end if;
 	end if;
end process;
		
--// AUX ADC SECTION ---------------
-- Automatically initiate an ADC conversion request upon finishing reading 
-- the 4 AUX ADCs.
-- AUX_ADC conversion request. Queue until conversion starts
AUX_ADC_001: process(CLK)
begin
	if rising_edge(CLK) then
		if (INITIAL_CONFIGURATION_COMPLETE = '1') and (CLK_DIV = '1') and (SPI_ADDR = 32) and (SPI_BUSY = '0')  then
			-- wait until SPI read of address 33 is complete
			AUX_ADC_CONVERT_REQ_PENDING <= '1';
			AUX_ADC_1_2N <= not AUX_ADC_1_2N;	
			-- 1/2 mux selection alternates every other conversion.
		elsif (INITIAL_CONFIGURATION_COMPLETE = '1')  and (SPI_ADDR = 34) and (SPI_BUSY = '0')  then
			-- just started configuring register 34. Ready for another word.
			AUX_ADC_CONVERT_REQ_PENDING <= '0';
		end if;
 	end if;
end process;

-- Make sure there is a sufficient delay after start of conversion. 
-- Not necessary. (keep code just in case)
--HOLD_AUX_ADC_READ_001: process(CLK)
--begin
--	if rising_edge(CLK) then
--		if (INITIAL_CONFIGURATION_COMPLETE = '1') and (CLK_DIV = '1') and (SPI_ADDR = 32) and (SPI_BUSY = '0')  then
--			-- load counter (same start criteria as for AUX_ADC_CONVERT_REQ_PENDING)
--			HOLD_AUX_ADC_READ_COUNTER <= "111111111";
--		elsif (HOLD_AUX_ADC_READ_COUNTER /= 0) and (CLK_DIV = '1') then
--			-- decrement counter until zero
--			HOLD_AUX_ADC_READ_COUNTER <= HOLD_AUX_ADC_READ_COUNTER + "111111111";
--		end if;
-- 	end if;
--end process;

--HOLD_AUX_ADC_READ <= '0' when (HOLD_AUX_ADC_READ_COUNTER = 0) else '1';

-- Reconstruct the ADC value from two bytes fragments
AUX_ADC_002: process(CLK)
begin
	if rising_edge(CLK) then
		if(SPI_ADDR = 26) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC A2 data(1:0)
			AUX_ADC_A2(1 downto 0) <= SPI_DATA_IN(7 downto 6);
		elsif(SPI_ADDR = 27) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC A2 data(9:2)
			AUX_ADC_A2(9 downto 2) <= SPI_DATA_IN;
			AUX_ADC_A2_SAMPLE_CLK <= '1';	-- data is ready to be read
		elsif(SPI_ADDR = 28) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC A1 data(1:0)
			AUX_ADC_A1(1 downto 0) <= SPI_DATA_IN(7 downto 6);
		elsif(SPI_ADDR = 29) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC A1 data(9:2)
			AUX_ADC_A1(9 downto 2) <= SPI_DATA_IN;
			AUX_ADC_A1_SAMPLE_CLK <= '1';	-- data is ready to be read
		elsif(SPI_ADDR = 30) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC B2 data(1:0)
			AUX_ADC_B2(1 downto 0) <= SPI_DATA_IN(7 downto 6);
		elsif(SPI_ADDR = 31) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC B2 data(9:2)
			AUX_ADC_B2(9 downto 2) <= SPI_DATA_IN;
			AUX_ADC_B2_SAMPLE_CLK <= '1';	-- data is ready to be read
		elsif(SPI_ADDR = 32) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC B1 data(1:0)
			AUX_ADC_B1(1 downto 0) <= SPI_DATA_IN(7 downto 6);
		elsif(SPI_ADDR = 33) and (SPI_DATA_IN_CLK = '1') then
			-- AUX ADC B1 data(9:2)
			AUX_ADC_B1(9 downto 2) <= SPI_DATA_IN;
			AUX_ADC_B1_SAMPLE_CLK <= '1';	-- data is ready to be read
		else
			AUX_ADC_A2_SAMPLE_CLK <= '0';	
			AUX_ADC_A1_SAMPLE_CLK <= '0';	
			AUX_ADC_B2_SAMPLE_CLK <= '0';	
			AUX_ADC_B1_SAMPLE_CLK <= '0';	
		end if;
 	end if;
end process;

--// RX GAIN SECTION
RX_GAIN_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(RxPGA_SAMPLE_CLK = '1') then
			-- wait until SPI write is complete
			RX_GAIN_CONFIG_PENDING <= '1';
		elsif (INITIAL_CONFIGURATION_COMPLETE = '1') and (CLK_DIV = '1') and (SPI_ADDR = 3) and (SPI_BUSY = '0')  then
			-- just started configuring register 3. Ready for another word.
			RX_GAIN_CONFIG_PENDING <= '0';
		end if;
 	end if;
end process;

--// SPI INTERFACE ---------------
-- Connect SPI module to SPI interface
-- Latency between SPI_WRITE_REQ and SPI_BUSY is 0 CLK
-- Latency between SPI_READ_REQ and SPI_BUSY is 0 CLK
-- Maximum speed is 16 MHz.
SPI_001: SPI port map(
	ASYNC_RESET => ASYNC_RESET, 
	CLK => CLK,
	CLK_DIV => CLK_DIV,
	ADDR => SPI_ADDR,
	DATA_OUT => SPI_DATA_OUT,
	WRITE_REQ => SPI_WRITE_REQ,
	READ_REQ => SPI_READ_REQ,
	DATA_IN => SPI_DATA_IN,
	DATA_IN_CLK => SPI_DATA_IN_CLK,
	BUSY => SPI_BUSY,
	SEN => AFE_CNTRL4,
	SCLK => AFE_CNTRL1,
	SDIO => AFE_CNTRL3,
	SDO => AFE_CNTRL2
	);

--// AUX SPI (unused)
AUX_SPI <= "000";

--// Receiver section
-- Reclock input samples at falling edge of AFE_CNTRL5.
-- Place in dual port buffer. Generate write pointer.
-- Read synchronously with CLK.

RECLOCK_AFE_RX_DATA_001: process(AFE_CNTRL5)
begin
	if falling_edge(AFE_CNTRL5) then
		AFE_RX_DATA_A_D(11 downto 0) <= AFE_RX_DATA_A;
		AFE_RX_DATA_B_D(11 downto 0) <= AFE_RX_DATA_B;
		-- no flow control for write: 
		-- Key assumption: CLK is faster than AFE_CNTRL5
		RX_WPTR <= RX_WPTR + 1;
	end if;
end process;


RAMB4_S16_S16_001: RAMB4_S16_S16 port map(
	DIA => AFE_RX_DATA_A_D,
	DIB => ZERO16,
	ENA => ONE,
	ENB => ONE,
	WEA => ONE,				-- new sample at each AFE_CNTRL5 clock period
	WEB => ZERO,
	RSTA => ASYNC_RESET,
	RSTB => ASYNC_RESET,
	CLKA => AFE_CNTRL5_N,	-- i.e. falling edge of AFE_CNTRL5
	CLKB => CLK,
	ADDRA => RX_WPTR,
	ADDRB => RX_RPTR,
	DOB => RX_DATA_A_E
	);

RAMB4_S16_S16_002: RAMB4_S16_S16 port map(
	DIA => AFE_RX_DATA_B_D,
	DIB => ZERO16,
	ENA => ONE,
	ENB => ONE,
	WEA => ONE,				-- new sample at each AFE_CNTRL5 clock period
	WEB => ZERO,
	RSTA => ASYNC_RESET,
	RSTB => ASYNC_RESET,
	CLKA => AFE_CNTRL5_N,	-- i.e. falling edge of AFE_CNTRL5
	CLKB => CLK,
	ADDRA => RX_WPTR,
	ADDRB => RX_RPTR,
	DOB => RX_DATA_B_E
	);


-- generate RX_WPTR replica synchronous with faster CLK_P.
-- Detect change by checking changes in RX_WPTR(0) [15.6ns], thus always detected by 90 MHz clock
RX_WPTR_GEN_002: process(CLK, RX_WPTR)
begin
	if rising_edge(CLK) then
		RX_WPTR_0_D <= RX_WPTR(0);
		RX_WPTR_0_D2 <= RX_WPTR_0_D;

		if(RX_WPTR_0_D2 /= RX_WPTR_0_D) then
			-- RX_WPTR was incremented
			RX_WPTR_P <= RX_WPTR_P + 1;
		end if;
	end if;
end process;

-- Buffer contents size is (write pointer - read pointer)
-- affected by glitches because AFE clock and CLK are asynchronous
-- However, thresholds are far apart to avoid any problems.
RX_BUF_EMPTY <= '1' when (RX_WPTR_P = RX_RPTR_INC) else '0';
RX_RPTR_INC <= RX_RPTR + 1;

RX_RPTR_GEN_001: process(ASYNC_RESET, CLK)
begin
	if(ASYNC_RESET = '1') then
		RX_RPTR <= (others => '1');
	elsif rising_edge(CLK) then
		-- take a clock for the DPRAM to generate the data after a change of address
		RX_DATA_SAMPLE_CLK <= RX_DATA_READ_REQ;

		if(RX_BUF_EMPTY = '0') then
			-- flow control for read. 
			-- Key assumption: CLK is faster than AFE_CNTRL5
			RX_RPTR <= RX_RPTR_INC;
			RX_DATA_READ_REQ <= '1';
		else
			RX_DATA_READ_REQ <= '0';
		end if;
	end if;
end process;

-- Keep only the lower 12-bits
RX_DATA_A <= RX_DATA_A_E(11 downto 0);
RX_DATA_B <= RX_DATA_B_E(11 downto 0);

--// Transmit section
-- Time critical section. Key assumption is that CLK is always noticeably faster
-- than the 64Msamples/s complex output sampling clock.

-- use A/B/C buffer
RECLOCK_002: process(CLK)
begin
	if rising_edge(CLK) then
		if(TX_SELECT < 2) then
			TX_SELECT <= TX_SELECT + 1;
		else 
			TX_SELECT <= 0;
		end if;

		if(TX_SELECT = 0) then
			TX_DATA_A_0 <= TX_DATA_A;
			TX_DATA_B_0 <= TX_DATA_B;
			TX_SELECT0 <= '1';
			TX_SELECT2 <= '0';
		elsif(TX_SELECT = 1) then
			TX_DATA_A_1 <= TX_DATA_A;
			TX_DATA_B_1 <= TX_DATA_B;
			TX_SELECT0 <= '0';
			TX_SELECT1 <= '1';
		else
			TX_DATA_A_2 <= TX_DATA_A;
			TX_DATA_B_2 <= TX_DATA_B;
			TX_SELECT1 <= '0';
			TX_SELECT2 <= '1';
		end if;
	end if;
end process;

-- reclock TX_SELECT_X with CLKOUT2B
RECLOCK_003: process(CLKOUT2B, TX_SELECT0, TX_SELECT1, TX_SELECT2)
begin
	if falling_edge(CLKOUT2B) then
		TX_SELECT0_D <=  TX_SELECT0;
		TX_SELECT1_D <=  TX_SELECT1;
		TX_SELECT2_D <=  TX_SELECT2;
	end if;
end process;

-- reclock with 64 MHz transmit sampling clock (128 MHz / 2)
-- Either the 0, 1 or 2 buffer is stable stable during resampling
RECLOCK_004: process(CLKOUT2B, TX_DATA_A_0, TX_DATA_A_1, TX_DATA_A_2, TX_DATA_B_0,
	TX_DATA_B_1, TX_DATA_B_2)
begin
	if falling_edge(CLKOUT2B) then
		AFE_TXSYNC_LOCAL <= not AFE_TXSYNC_LOCAL;

		if(AFE_TXSYNC_LOCAL = '0') then
			if(TX_SELECT1_D = '1') then
				AFE_TX_DATA_A <= TX_DATA_A_0;
				AFE_TX_DATA_B <= TX_DATA_B_0;
			elsif(TX_SELECT2_D = '1') then
				AFE_TX_DATA_A <= TX_DATA_A_1;
				AFE_TX_DATA_B <= TX_DATA_B_1;
			else
			--elsif(TX_SELECT0_D = '1') then
				AFE_TX_DATA_A <= TX_DATA_A_2;
				AFE_TX_DATA_B <= TX_DATA_B_2;
			end if;

		end if;
	end if;
end process;

S1_3 <= AFE_TX_DATA_A(13 downto 6);
S1_3_CLK <= '1';
S2_3 <= AFE_TX_DATA_B(13 downto 6);
S2_3_CLK <= '1';

-- multiplex A/B channels at 128 MHz
MUX_OUT_GEN_001: process(CLKOUT2B)
begin
	if falling_edge(CLKOUT2B) then
		if(AFE_TXSYNC_LOCAL = '1') then
			-- send A channel together with TXSYNC = '0'
			AFE_TX_DATA <= AFE_TX_DATA_A;
			AFE_TXSYNC <= '0';	
				-- trick: VHDL writing style is akward.. but the point is to delay AFE_TXSYNC_LOCAL
		else
			-- send B channel together with TXSYNC = '1'
			AFE_TX_DATA <= AFE_TX_DATA_B;
			AFE_TXSYNC <= '1';	
		end if;
	end if;
end process;



-- because of the speed, one has to massage the constraint file
-- to minimize the overall output delay of the 14-bit muxed word.


--// Test points AFE
TP(2) <= SPI_READ_REQ;
TP(1) <= SPI_BUSY;
TP(3) <= SPI_WRITE_REQ;
TP(4) <= '1' when (SPI_ADDR = 26) else '0';
TP(5) <= '1' when (SPI_ADDR = 32) else '0';
TP(6) <= '1' when (SPI_ADDR = 33) else '0';
TP(7) <= '1' when (SPI_ADDR = 34) else '0';
TP(8) <= AFE_CNTRL2;
TP(9) <= AUX_ADC_CONVERT_REQ_PENDING;
TP(10) <= AUX_ADC_1_2N;

end behavioral;